/* Copyright 2015 Link Labs LLC.  All rights reserved. */

#include "str_utils.h"
#include "embc/dbc.h"
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

void cmd_reformat(char * s)
{
    while (s && *s)
    {
        if ((*s >= 'A') && (*s <= 'Z'))
        {   // force lower case
            *s = (*s - 'A') + 'a';
        }
        if (*s == '_')
        {   // underscores are dashes
            *s = '-';
        }
        ++s;
    }
}

int str_to_int32(char const * value_str, int32_t * value)
{
    uint32_t v = 0;
    int rc = 0;

    DBC_ARG_NOT_NULL(value);
    rc = str_to_uint32(value_str, &v);
    if (rc)
    {
        return rc;
    }
    *value = (int32_t) v;
    return 0;
}

int str_to_uint32(char const * value_str, uint32_t * value)
{
    char * endptr = 0;
    DBC_ARG_NOT_NULL(value_str);
    DBC_ARG_NOT_NULL(value);

    if ((value_str[0] == '0') && (value_str[1] == 'x'))
    {
        *value = (uint32_t) strtol(value_str + 2, &endptr, 16);
    }
    else
    {
        *value = (uint32_t) strtol(value_str, &endptr, 10);
    }
    if (*endptr)  // not all characters used!
    {
        return 1;
    }
    return 0;
}

int str_to_uint8(char const * value_str, uint8_t * value)
{
    uint32_t v32 = 0;
    int rc = str_to_uint32(value_str, &v32);
    if (rc)
    {
        return rc;
    }
    if (v32 > 255)
    {
        return 2;
    }
    *value = (uint8_t) v32;
    return 0;
}

int32_t hex_char_to_value(char ch, uint8_t *value)
{
    DBC_ARG_NOT_NULL(value);
    if ((ch >= '0') && (ch <= '9'))
    {
        *value = ch - '0';
        return 0;
    }
    else if ((ch >= 'A') && (ch <= 'F'))
    {
        *value = ch - 'A' + 10;
        return 0;
    }
    else if ((ch >= 'a') && (ch <= 'f'))
    {
        *value = ch - 'a' + 10;
        return 0;
    }

    return 1;
}

int32_t hex_string_to_bytes(char const * hex_str, uint8_t * b, size_t sz, size_t * sz_actual)
{
    char const * s = hex_str;
    size_t nibbles = 0;
    uint8_t v;

    DBC_ARG_NOT_NULL(hex_str);
    DBC_ARG_NOT_NULL(b);

    if ((0 == hex_str) || (0 == b) || (0 == sz)) {
        return -1;
    }
    while (*s)
    {
        if ((nibbles >> 1) >= sz)
        {
            LOGF_ERROR("string too long: %s", hex_str);
            return -1;
        }
        if (strchr(" ,._-\t", *s))
        {
            ++s;
            continue;
        }
        if (hex_char_to_value(*s++, &v))
        {
            return 1;
        }
        if (nibbles & 1)
        {
            *b |= v;
            ++b;
        }
        else
        {
            *b = (v << 4);
        }
        ++nibbles;
    }
    if (nibbles & 1)
    {
        LOGS_WARN("odd number of hex characters");
    }
    if (sz_actual)
    {
        *sz_actual = (nibbles + 1) >> 1;
    }
    return 0;
}

int safestr_toupper(char * s) {
    DBC_ARG_NOT_NULL(s);

    while (*s) {
        *s = toupper((uint8_t) *s);
        ++s;
    }
    return 0;
}

int safestr_toIndex(char const * s, char const * const * table, int * index) {
    int idx = 0;

    DBC_ARG_NOT_NULL(s);
    DBC_ARG_NOT_NULL(table);
    DBC_ARG_NOT_NULL(index);

    while (*table) {
        if (strcmp(s, *table) == 0) {
            *index = idx;
            return EMBC_SUCCESS;
        }
        ++idx;
        ++table;
    }
    return 1;
}

